"""Using depends() to mark dependencies at runtime.
"""

import pytest


def test_skip_depend_runtime(ctestdir):
    """One test is skipped, other dependent tests are skipped as well.
    This also includes indirect dependencies.
    """
    ctestdir.makepyfile("""
        import pytest
        from pytest_dependency import depends

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pytest.skip("explicit skip")

        @pytest.mark.dependency()
        def test_c(request):
            depends(request, ["test_b"])
            pass

        @pytest.mark.dependency()
        def test_d(request):
            depends(request, ["test_a", "test_c"])
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1, skipped=3, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b SKIPPED
        *::test_c SKIPPED
        *::test_d SKIPPED
    """)



import pytest

# Test data
# Consider a bunch of Nodes, some of them are parents and some are children.

class Node(object):
    NodeMap = {}
    def __init__(self, name, parent=None):
        self.name = name
        self.children = []
        self.NodeMap[self.name] = self
        if parent:
            self.parent = self.NodeMap[parent]
            self.parent.children.append(self)
        else:
            self.parent = None
    def __str__(self):
        return self.name

parents = [ Node("a"),  Node("b"),  Node("c"),  Node("d"), ]
childs =  [ Node("e", "a"), Node("f", "a"), Node("g", "a"), 
            Node("h", "b"), Node("i", "c"), Node("j", "c"), 
            Node("k", "d"), Node("l", "d"), Node("m", "d"), ]

# The test for the parent shall depend on the test of all its children.
# Create enriched parameter lists, decorated with the dependency marker.

childparam = [ 
    pytest.param(c, marks=pytest.mark.dependency(name="test_child[%s]" % c)) 
    for c in childs
]
parentparam = [
    pytest.param(p, marks=pytest.mark.dependency(
        name="test_parent[%s]" % p, 
        depends=["test_child[%s]" % c for c in p.children]
    )) for p in parents
]

@pytest.mark.parametrize("c", childparam)
def test_child(c):
    if c.name == "l":
        pytest.xfail("deliberate fail")
        assert False

@pytest.mark.parametrize("p", parentparam)
def test_parent(p):
    pass




import pytest

def instances(name, params):
    def vstr(val):
        if isinstance(val, (list, tuple)):
            return "-".join([str(v) for v in val])
        else:
            return str(val)
    return ["%s[%s]" % (name, vstr(v)) for v in params]


params_a = range(17)

@pytest.mark.parametrize("x", params_a)
@pytest.mark.dependency()
def test_a(x):
    if x == 13:
        pytest.xfail("deliberate fail")
        assert False
    else:
        pass

@pytest.mark.dependency(depends=instances("test_a", params_a))
def test_b():
    pass

params_c = list(zip(range(0,8,2), range(2,6)))

@pytest.mark.parametrize("x,y", params_c)
@pytest.mark.dependency()
def test_c(x, y):
    if x > y:
        pytest.xfail("deliberate fail")
        assert False
    else:
        pass

@pytest.mark.dependency(depends=instances("test_c", params_c))
def test_d():
    pass

params_e = ['abc', 'def']

@pytest.mark.parametrize("s", params_e)
@pytest.mark.dependency()
def test_e(s):
    if 'e' in s:
        pytest.xfail("deliberate fail")
        assert False
    else:
        pass

@pytest.mark.dependency(depends=instances("test_e", params_e))
def test_f():
    pass



import pytest

pytest_plugins = "pytester"


@pytest.fixture
def ctestdir(testdir):
    testdir.makefile('.ini', pytest="""
        [pytest]
        console_output_style = classic
    """)
    testdir.makeconftest("""
        import sys
        if "pytest_dependency" not in sys.modules:
            pytest_plugins = "pytest_dependency"
    """)
    return testdir



import pytest
from pytest_dependency import depends

@pytest.fixture(scope="module", params=range(1,10))
def testcase(request):
    param = request.param
    return param

@pytest.mark.dependency()
def test_a(testcase):
    if testcase % 7 == 0:
        pytest.xfail("deliberate fail")
        assert False

@pytest.mark.dependency()
def test_b(request, testcase):
    depends(request, ["test_a[%d]" % testcase])
    pass



"""A scenario featuring parametrized tests.
"""

import pytest


def test_multiple(ctestdir):
    ctestdir.makepyfile("""
        import pytest

        _md = pytest.mark.dependency

        @pytest.mark.parametrize("x,y", [
            pytest.param(0, 0, marks=_md(name="a1")),
            pytest.param(0, 1, marks=_md(name="a2")),
            pytest.param(1, 0, marks=_md(name="a3")),
            pytest.param(1, 1, marks=_md(name="a4"))
        ])
        def test_a(x,y):
            assert x==0 or y==0

        @pytest.mark.parametrize("u,v", [
            pytest.param(1, 2, marks=_md(name="b1", depends=["a1", "a2"])),
            pytest.param(1, 3, marks=_md(name="b2", depends=["a1", "a3"])),
            pytest.param(1, 4, marks=_md(name="b3", depends=["a1", "a4"])),
            pytest.param(2, 3, marks=_md(name="b4", depends=["a2", "a3"])),
            pytest.param(2, 4, marks=_md(name="b5", depends=["a2", "a4"])),
            pytest.param(3, 4, marks=_md(name="b6", depends=["a3", "a4"]))
        ])
        def test_b(u,v):
            pass

        @pytest.mark.parametrize("w", [
            pytest.param(1, marks=_md(name="c1", depends=["b1", "b3", "b5"])),
            pytest.param(2, marks=_md(name="c2", depends=["b1", "b3", "b6"])),
            pytest.param(3, marks=_md(name="c3", depends=["b1", "b2", "b4"]))
        ])
        def test_c(w):
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=7, skipped=5, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a?0-0? PASSED
        *::test_a?0-1? PASSED
        *::test_a?1-0? PASSED
        *::test_a?1-1? FAILED
        *::test_b?1-2? PASSED
        *::test_b?1-3? PASSED
        *::test_b?1-4? SKIPPED
        *::test_b?2-3? PASSED
        *::test_b?2-4? SKIPPED
        *::test_b?3-4? SKIPPED
        *::test_c?1? SKIPPED
        *::test_c?2? SKIPPED
        *::test_c?3? PASSED
    """)



# test_mod_02.py

import pytest

@pytest.mark.dependency()
@pytest.mark.xfail(reason="deliberate fail")
def test_a():
    assert False

@pytest.mark.dependency(
    depends=["tests/test_mod_01.py::test_a", "tests/test_mod_01.py::test_c"],
    scope='session'
)
def test_e():
    pass

@pytest.mark.dependency(
    depends=["tests/test_mod_01.py::test_b", "tests/test_mod_02.py::test_e"],
    scope='session'
)
def test_f():
    pass

@pytest.mark.dependency(
    depends=["tests/test_mod_01.py::TestClass::test_b"],
    scope='session'
)
def test_g():
    pass



"""Test the ignore-unknown-dependency command line option.
"""

import pytest


def test_no_ignore(ctestdir):
    """No command line option, e.g. ignore-unknown-dependency is not set.

    Explicitly select only a single test that depends on another one.
    Since the other test has not been run at all, the selected test
    will be skipped.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "test_no_ignore.py::test_d")
    result.assert_outcomes(passed=0, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_d SKIPPED
    """)


def test_ignore(ctestdir):
    """Set the ignore-unknown-dependency command line option.

    Explicitly select only a single test that depends on another one.
    The other test has not been run at all, but since unknown
    dependencies will be ignored, the selected test will be run
    nevertheless.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "--ignore-unknown-dependency", 
                                "test_ignore.py::test_d")
    result.assert_outcomes(passed=1, skipped=0, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_d PASSED
    """)



"""Usage with test classes.
"""

import pytest


def test_class_simple(ctestdir):
    """Simple dependencies of test methods in a class.
    test_a() deliberately fails, some other methods depend on it, some don't.
    """
    ctestdir.makepyfile("""
        import pytest

        class TestClass(object):

            @pytest.mark.dependency()
            def test_a(self):
                assert False

            @pytest.mark.dependency()
            def test_b(self):
                pass

            @pytest.mark.dependency(depends=["TestClass::test_a"])
            def test_c(self):
                pass

            @pytest.mark.dependency(depends=["TestClass::test_b"])
            def test_d(self):
                pass

            @pytest.mark.dependency(depends=["TestClass::test_b", 
                                             "TestClass::test_c"])
            def test_e(self):
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=2, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        *::TestClass::test_a FAILED
        *::TestClass::test_b PASSED
        *::TestClass::test_c SKIPPED
        *::TestClass::test_d PASSED
        *::TestClass::test_e SKIPPED
    """)


def test_class_simple_named(ctestdir):
    """Mostly the same as test_class_simple(), but name the test methods
    now explicitely.
    """
    ctestdir.makepyfile("""
        import pytest

        class TestClassNamed(object):

            @pytest.mark.dependency(name="a")
            def test_a(self):
                assert False

            @pytest.mark.dependency(name="b")
            def test_b(self):
                pass

            @pytest.mark.dependency(name="c", depends=["a"])
            def test_c(self):
                pass

            @pytest.mark.dependency(name="d", depends=["b"])
            def test_d(self):
                pass

            @pytest.mark.dependency(name="e", depends=["b", "c"])
            def test_e(self):
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=2, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        *::TestClassNamed::test_a FAILED
        *::TestClassNamed::test_b PASSED
        *::TestClassNamed::test_c SKIPPED
        *::TestClassNamed::test_d PASSED
        *::TestClassNamed::test_e SKIPPED
    """)


def test_class_default_name(ctestdir):
    """Issue #6: for methods of test classes, the default name used to be
    the method name.  This could have caused conflicts if there is a
    function having the same name outside the class.  In the following
    example, before fixing this issue, the method test_a() of class
    TestClass would have shadowed the failure of function test_a().

    Now the class name is prepended to the default test name, removing
    this conflict.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            assert False

        class TestClass(object):

            @pytest.mark.dependency()
            def test_a(self):
                pass

        @pytest.mark.dependency(depends=["test_a"])
        def test_b():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1, skipped=1, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a FAILED
        *::TestClass::test_a PASSED
        *::test_b SKIPPED
    """)



import pytest

@pytest.mark.dependency(name="a")
@pytest.mark.xfail(reason="deliberate fail")
def test_a():
    assert False

@pytest.mark.dependency(name="b")
def test_b():
    pass

@pytest.mark.dependency(name="c", depends=["a"])
def test_c():
    pass

@pytest.mark.dependency(name="d", depends=["b"])
def test_d():
    pass

@pytest.mark.dependency(name="e", depends=["b", "c"])
def test_e():
    pass



import random
import pytest

def test_a():
    pass

@pytest.mark.parametrize("i,b", [
    (7, True),
    (0, False),
    pytest.param(-1, False, marks=pytest.mark.xfail(reason="nonsense"))
])
def test_b(i, b):
    assert bool(i) == b

ordered = list(range(10))
unordered = random.sample(ordered, k=len(ordered))

class TestClass:

    def test_c(self):
        pass

    @pytest.mark.parametrize("l,ll", [(ordered, 10), (unordered, 10)],
                             ids=["order", "disorder"])
    def test_d(self, l, ll):
        assert len(l) == ll



import pytest

@pytest.mark.parametrize("x,y", [
    pytest.param(0, 0, marks=pytest.mark.dependency(name="a1")),
    pytest.param(0, 1, marks=[pytest.mark.dependency(name="a2"),
                              pytest.mark.xfail]),
    pytest.param(1, 0, marks=pytest.mark.dependency(name="a3")),
    pytest.param(1, 1, marks=pytest.mark.dependency(name="a4"))
])
def test_a(x,y):
    assert y <= x

@pytest.mark.parametrize("u,v", [
    pytest.param(1, 2, marks=pytest.mark.dependency(name="b1", 
                                                    depends=["a1", "a2"])),
    pytest.param(1, 3, marks=pytest.mark.dependency(name="b2", 
                                                    depends=["a1", "a3"])),
    pytest.param(1, 4, marks=pytest.mark.dependency(name="b3", 
                                                    depends=["a1", "a4"])),
    pytest.param(2, 3, marks=pytest.mark.dependency(name="b4", 
                                                    depends=["a2", "a3"])),
    pytest.param(2, 4, marks=pytest.mark.dependency(name="b5", 
                                                    depends=["a2", "a4"])),
    pytest.param(3, 4, marks=pytest.mark.dependency(name="b6", 
                                                    depends=["a3", "a4"]))
])
def test_b(u,v):
    pass

@pytest.mark.parametrize("w", [
    pytest.param(1, marks=pytest.mark.dependency(name="c1", 
                                                 depends=["b1", "b2", "b6"])),
    pytest.param(2, marks=pytest.mark.dependency(name="c2", 
                                                 depends=["b2", "b3", "b6"])),
    pytest.param(3, marks=pytest.mark.dependency(name="c3", 
                                                 depends=["b2", "b4", "b6"]))
])
def test_c(w):
    pass



"""Verify the messages issued when a dependent test is skipped.
"""

import pytest


def test_simple(ctestdir):
    """One test fails, other dependent tests are skipped.
    This also includes indirect dependencies.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            assert False

        @pytest.mark.dependency(depends=["test_b"])
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "-rs")
    result.assert_outcomes(passed=1, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b FAILED
        *::test_c SKIPPED
        *::test_d SKIPPED
    """)
    result.stdout.fnmatch_lines_random("""
        SKIP* test_c depends on test_b
        SKIP* test_d depends on test_c
    """)



import pytest
from pytest_dependency import depends

@pytest.mark.dependency()
def test_a():
    pass

@pytest.mark.dependency()
@pytest.mark.xfail(reason="deliberate fail")
def test_b():
    assert False

@pytest.mark.dependency()
def test_c(request):
    depends(request, ["test_b"])
    pass

@pytest.mark.dependency()
def test_d(request):
    depends(request, ["test_a", "test_c"])
    pass



"""Specifying the scope of dependencies.
"""

import pytest


def test_scope_module(ctestdir):
    """One single module, module scope is explicitely set in the
    pytest.mark.dependency() marker.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            assert False

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency(depends=["test_a"], scope='module')
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_b"], scope='module')
        def test_d():
            pass

        @pytest.mark.dependency(depends=["test_b", "test_c"], scope='module')
        def test_e():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=2, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        test_scope_module.py::test_a FAILED
        test_scope_module.py::test_b PASSED
        test_scope_module.py::test_c SKIPPED
        test_scope_module.py::test_d PASSED
        test_scope_module.py::test_e SKIPPED
    """)

def test_scope_session(ctestdir):
    """Two modules, some cross module dependencies in session scope.
    """
    ctestdir.makepyfile(test_scope_session_01="""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            assert False

        @pytest.mark.dependency(depends=["test_a"])
        def test_c():
            pass

        class TestClass(object):

            @pytest.mark.dependency()
            def test_b(self):
                pass
    """, test_scope_session_02="""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            assert False

        @pytest.mark.dependency(
            depends=["test_scope_session_01.py::test_a",
                     "test_scope_session_01.py::test_c"],
            scope='session'
        )
        def test_e():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_session_01.py::test_b"],
            scope='session'
        )
        def test_f():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_session_02.py::test_e"],
            scope='session'
        )
        def test_g():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_session_01.py::TestClass::test_b"],
            scope='session'
        )
        def test_h():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=6, skipped=1, failed=2)
    result.stdout.fnmatch_lines("""
        test_scope_session_01.py::test_a PASSED
        test_scope_session_01.py::test_b FAILED
        test_scope_session_01.py::test_c PASSED
        test_scope_session_01.py::TestClass::test_b PASSED
        test_scope_session_02.py::test_a FAILED
        test_scope_session_02.py::test_e PASSED
        test_scope_session_02.py::test_f SKIPPED
        test_scope_session_02.py::test_g PASSED
        test_scope_session_02.py::test_h PASSED
    """)

def test_scope_package(ctestdir):
    """Two packages, some cross module dependencies within the package and
    across package boundaries.
    """
    ctestdir.mkpydir("test_scope_package_a")
    ctestdir.mkpydir("test_scope_package_b")
    srcs = {
        'test_scope_package_a/test_01': """
            import pytest

            @pytest.mark.dependency()
            def test_a():
                pass
        """,
        'test_scope_package_b/test_02': """
            import pytest

            @pytest.mark.dependency()
            def test_c():
                pass

            @pytest.mark.dependency()
            def test_d():
                assert False
        """,
        'test_scope_package_b/test_03': """
            import pytest

            @pytest.mark.dependency(
                depends=["test_scope_package_a/test_01.py::test_a"],
                scope='session'
            )
            def test_e():
                pass

            @pytest.mark.dependency(
                depends=["test_scope_package_a/test_01.py::test_a"],
                scope='package'
            )
            def test_f():
                pass

            @pytest.mark.dependency(
                depends=["test_scope_package_b/test_02.py::test_c"],
                scope='package'
            )
            def test_g():
                pass

            @pytest.mark.dependency(
                depends=["test_scope_package_b/test_02.py::test_d"],
                scope='package'
            )
            def test_h():
                pass
        """,
    }
    ctestdir.makepyfile(**srcs)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=4, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        test_scope_package_a/test_01.py::test_a PASSED
        test_scope_package_b/test_02.py::test_c PASSED
        test_scope_package_b/test_02.py::test_d FAILED
        test_scope_package_b/test_03.py::test_e PASSED
        test_scope_package_b/test_03.py::test_f SKIPPED
        test_scope_package_b/test_03.py::test_g PASSED
        test_scope_package_b/test_03.py::test_h SKIPPED
    """)

def test_scope_class(ctestdir):
    """Dependencies in class scope.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            assert False

        @pytest.mark.dependency()
        def test_b():
            pass

        class TestClass1(object):

            @pytest.mark.dependency()
            def test_c(self):
                pass

        class TestClass2(object):

            @pytest.mark.dependency()
            def test_a(self):
                pass

            @pytest.mark.dependency()
            def test_b(self):
                assert False

            @pytest.mark.dependency(depends=["test_a"])
            def test_d(self):
                pass

            @pytest.mark.dependency(depends=["test_b"])
            def test_e(self):
                pass

            @pytest.mark.dependency(depends=["test_a"], scope='class')
            def test_f(self):
                pass

            @pytest.mark.dependency(depends=["test_b"], scope='class')
            def test_g(self):
                pass

            @pytest.mark.dependency(depends=["test_c"], scope='class')
            def test_h(self):
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=5, skipped=3, failed=2)
    result.stdout.fnmatch_lines("""
        test_scope_class.py::test_a FAILED
        test_scope_class.py::test_b PASSED
        test_scope_class.py::TestClass1::test_c PASSED
        test_scope_class.py::TestClass2::test_a PASSED
        test_scope_class.py::TestClass2::test_b FAILED
        test_scope_class.py::TestClass2::test_d SKIPPED
        test_scope_class.py::TestClass2::test_e PASSED
        test_scope_class.py::TestClass2::test_f PASSED
        test_scope_class.py::TestClass2::test_g SKIPPED
        test_scope_class.py::TestClass2::test_h SKIPPED
    """)

def test_scope_nodeid(ctestdir):
    """The default name of a test is the node id.
    The references to the default names must be adapted according to
    the scope.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency(
            depends=["test_a"],
            scope='module'
        )
        def test_b():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_nodeid.py::test_a"],
            scope='module'
        )
        def test_c():
            pass

        @pytest.mark.dependency(
            depends=["test_a"],
            scope='session'
        )
        def test_d():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_nodeid.py::test_a"],
            scope='session'
        )
        def test_e():
            pass

        class TestClass(object):

            @pytest.mark.dependency()
            def test_f(self):
                pass

            @pytest.mark.dependency(
                depends=["test_f"],
                scope='class'
            )
            def test_g(self):
                pass

            @pytest.mark.dependency(
                depends=["TestClass::test_f"],
                scope='class'
            )
            def test_h(self):
                pass

            @pytest.mark.dependency(
                depends=["test_scope_nodeid.py::TestClass::test_f"],
                scope='class'
            )
            def test_i(self):
                pass

            @pytest.mark.dependency(
                depends=["test_f"],
                scope='module'
            )
            def test_j(self):
                pass

            @pytest.mark.dependency(
                depends=["TestClass::test_f"],
                scope='module'
            )
            def test_k(self):
                pass

            @pytest.mark.dependency(
                depends=["test_scope_nodeid.py::TestClass::test_f"],
                scope='module'
            )
            def test_l(self):
                pass

            @pytest.mark.dependency(
                depends=["test_f"],
                scope='session'
            )
            def test_m(self):
                pass

            @pytest.mark.dependency(
                depends=["TestClass::test_f"],
                scope='session'
            )
            def test_n(self):
                pass

            @pytest.mark.dependency(
                depends=["test_scope_nodeid.py::TestClass::test_f"],
                scope='session'
            )
            def test_o(self):
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=7, skipped=8, failed=0)
    result.stdout.fnmatch_lines("""
        test_scope_nodeid.py::test_a PASSED
        test_scope_nodeid.py::test_b PASSED
        test_scope_nodeid.py::test_c SKIPPED
        test_scope_nodeid.py::test_d SKIPPED
        test_scope_nodeid.py::test_e PASSED
        test_scope_nodeid.py::TestClass::test_f PASSED
        test_scope_nodeid.py::TestClass::test_g PASSED
        test_scope_nodeid.py::TestClass::test_h SKIPPED
        test_scope_nodeid.py::TestClass::test_i SKIPPED
        test_scope_nodeid.py::TestClass::test_j SKIPPED
        test_scope_nodeid.py::TestClass::test_k PASSED
        test_scope_nodeid.py::TestClass::test_l SKIPPED
        test_scope_nodeid.py::TestClass::test_m SKIPPED
        test_scope_nodeid.py::TestClass::test_n SKIPPED
        test_scope_nodeid.py::TestClass::test_o PASSED
    """)

def test_scope_named(ctestdir):
    """Explicitely named tests are always referenced by that name,
    regardless of the scope.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency(name="a")
        def test_a():
            pass

        @pytest.mark.dependency(
            depends=["a"],
            scope='module'
        )
        def test_b():
            pass

        @pytest.mark.dependency(
            depends=["test_a"],
            scope='module'
        )
        def test_c():
            pass

        @pytest.mark.dependency(
            depends=["a"],
            scope='session'
        )
        def test_d():
            pass

        @pytest.mark.dependency(
            depends=["test_scope_named.py::test_a"],
            scope='session'
        )
        def test_e():
            pass

        class TestClass(object):

            @pytest.mark.dependency(name="f")
            def test_f(self):
                pass

            @pytest.mark.dependency(
                depends=["f"],
                scope='class'
            )
            def test_g(self):
                pass

            @pytest.mark.dependency(
                depends=["test_f"],
                scope='class'
            )
            def test_h(self):
                pass

            @pytest.mark.dependency(
                depends=["f"],
                scope='module'
            )
            def test_i(self):
                pass

            @pytest.mark.dependency(
                depends=["TestClass::test_f"],
                scope='module'
            )
            def test_j(self):
                pass

            @pytest.mark.dependency(
                depends=["f"],
                scope='session'
            )
            def test_k(self):
                pass

            @pytest.mark.dependency(
                depends=["test_scope_named.py::TestClass::test_f"],
                scope='session'
            )
            def test_l(self):
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=7, skipped=5, failed=0)
    result.stdout.fnmatch_lines("""
        test_scope_named.py::test_a PASSED
        test_scope_named.py::test_b PASSED
        test_scope_named.py::test_c SKIPPED
        test_scope_named.py::test_d PASSED
        test_scope_named.py::test_e SKIPPED
        test_scope_named.py::TestClass::test_f PASSED
        test_scope_named.py::TestClass::test_g PASSED
        test_scope_named.py::TestClass::test_h SKIPPED
        test_scope_named.py::TestClass::test_i PASSED
        test_scope_named.py::TestClass::test_j SKIPPED
        test_scope_named.py::TestClass::test_k PASSED
        test_scope_named.py::TestClass::test_l SKIPPED
    """)

def test_scope_dependsfunc(ctestdir):
    """Test the scope argument to the depends() function.
    """
    ctestdir.makepyfile(test_scope_dependsfunc_01="""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            assert False

        @pytest.mark.dependency(depends=["test_a"])
        def test_c():
            pass

        class TestClass(object):

            @pytest.mark.dependency()
            def test_b(self):
                pass
    """, test_scope_dependsfunc_02="""
        import pytest
        from pytest_dependency import depends

        @pytest.mark.dependency()
        def test_a():
            assert False

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_e(request):
            depends(request,
                    ["test_scope_dependsfunc_01.py::test_a",
                     "test_scope_dependsfunc_01.py::test_c"],
                    scope='session')
            pass

        @pytest.mark.dependency()
        def test_f(request):
            depends(request,
                    ["test_scope_dependsfunc_01.py::test_b"],
                    scope='session')
            pass

        @pytest.mark.dependency()
        def test_g(request):
            depends(request,
                    ["test_scope_dependsfunc_02.py::test_e"],
                    scope='session')
            pass

        @pytest.mark.dependency()
        def test_h(request):
            depends(request,
                    ["test_scope_dependsfunc_01.py::TestClass::test_b"],
                    scope='session')
            pass

        @pytest.mark.dependency()
        def test_i(request):
            depends(request, ["test_a"], scope='module')
            pass

        @pytest.mark.dependency()
        def test_j(request):
            depends(request, ["test_b"], scope='module')
            pass

        class TestClass(object):

            @pytest.mark.dependency()
            def test_a(self):
                pass

            @pytest.mark.dependency()
            def test_b(self):
                assert False

            @pytest.mark.dependency()
            def test_c(self, request):
                depends(request, ["test_a"], scope='class')
                pass

            @pytest.mark.dependency()
            def test_d(self, request):
                depends(request, ["test_b"], scope='class')
                pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=10, skipped=3, failed=3)
    result.stdout.fnmatch_lines("""
        test_scope_dependsfunc_01.py::test_a PASSED
        test_scope_dependsfunc_01.py::test_b FAILED
        test_scope_dependsfunc_01.py::test_c PASSED
        test_scope_dependsfunc_01.py::TestClass::test_b PASSED
        test_scope_dependsfunc_02.py::test_a FAILED
        test_scope_dependsfunc_02.py::test_b PASSED
        test_scope_dependsfunc_02.py::test_e PASSED
        test_scope_dependsfunc_02.py::test_f SKIPPED
        test_scope_dependsfunc_02.py::test_g PASSED
        test_scope_dependsfunc_02.py::test_h PASSED
        test_scope_dependsfunc_02.py::test_i SKIPPED
        test_scope_dependsfunc_02.py::test_j PASSED
        test_scope_dependsfunc_02.py::TestClass::test_a PASSED
        test_scope_dependsfunc_02.py::TestClass::test_b FAILED
        test_scope_dependsfunc_02.py::TestClass::test_c PASSED
        test_scope_dependsfunc_02.py::TestClass::test_d SKIPPED
    """)



"""Test the automark_dependency option.
"""

import pytest


def test_not_set(ctestdir):
    """No pytest.ini file, e.g. automark_dependency is not set.

    Since automark_dependency defaults to false and test_a is not
    marked, the outcome of test_a will not be recorded.  As a result,
    test_b will be skipped due to a missing dependency.
    """
    ctestdir.makepyfile("""
        import pytest

        def test_a():
            pass

        @pytest.mark.dependency(depends=["test_a"])
        def test_b():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "-rs")
    result.assert_outcomes(passed=1, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b SKIPPED
    """)


def test_set_false(ctestdir):
    """A pytest.ini is present, automark_dependency is set to false.

    Since automark_dependency is set to false and test_a is not
    marked, the outcome of test_a will not be recorded.  As a result,
    test_b will be skipped due to a missing dependency.
    """
    ctestdir.makefile('.ini', pytest="""
        [pytest]
        automark_dependency = false
        console_output_style = classic
    """)
    ctestdir.makepyfile("""
        import pytest

        def test_a():
            pass

        @pytest.mark.dependency(depends=["test_a"])
        def test_b():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "-rs")
    result.assert_outcomes(passed=1, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b SKIPPED
    """)


def test_set_true(ctestdir):
    """A pytest.ini is present, automark_dependency is set to false.

    Since automark_dependency is set to true, the outcome of test_a
    will be recorded, even though it is not marked.  As a result,
    test_b will be skipped due to a missing dependency.
    """
    ctestdir.makefile('.ini', pytest="""
        [pytest]
        automark_dependency = true
        console_output_style = classic
    """)
    ctestdir.makepyfile("""
        import pytest

        def test_a():
            pass

        @pytest.mark.dependency(depends=["test_a"])
        def test_b():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "-rs")
    result.assert_outcomes(passed=2, skipped=0, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b PASSED
    """)



"""The most basic test: check that the marker works.
"""

import pytest


def test_marker_registered(ctestdir):
    result = ctestdir.runpytest("--markers")
    result.stdout.fnmatch_lines("""
        @pytest.mark.dependency*
    """)


def test_marker(ctestdir):
    ctestdir.makepyfile("""
        import pytest
        from pytest_dependency import DependencyManager

        @pytest.mark.dependency()
        def test_marker(request):
            node = request.node.getparent(pytest.Module)
            assert hasattr(node, 'dependencyManager')
            assert isinstance(node.dependencyManager, DependencyManager)
            assert 'test_marker' in node.dependencyManager.results
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1)



"""A complicated scenario with tests having multiple dependencies.
"""

import pytest


def test_multiple(ctestdir):
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency(name="a")
        def test_a():
            pytest.skip("explicit skip")

        @pytest.mark.dependency(name="b")
        def test_b():
            assert False

        @pytest.mark.dependency(name="c")
        def test_c():
            pass

        @pytest.mark.dependency(name="d")
        def test_d():
            pass

        @pytest.mark.dependency(name="e")
        def test_e():
            pass

        @pytest.mark.dependency(name="f", depends=["a", "c"])
        def test_f():
            pass

        @pytest.mark.dependency(name="g", depends=["b", "d"])
        def test_g():
            pass

        @pytest.mark.dependency(name="h", depends=["c", "e"])
        def test_h():
            pass

        @pytest.mark.dependency(name="i", depends=["f", "h"])
        def test_i():
            pass

        @pytest.mark.dependency(name="j", depends=["d", "h"])
        def test_j():
            pass

        @pytest.mark.dependency(name="k", depends=["g", "i", "j"])
        def test_k():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=5, skipped=5, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a SKIPPED
        *::test_b FAILED
        *::test_c PASSED
        *::test_d PASSED
        *::test_e PASSED
        *::test_f SKIPPED
        *::test_g SKIPPED
        *::test_h PASSED
        *::test_i SKIPPED
        *::test_j PASSED
        *::test_k SKIPPED
    """)



# test_mod_01.py

import pytest

@pytest.mark.dependency()
def test_a():
    pass

@pytest.mark.dependency()
@pytest.mark.xfail(reason="deliberate fail")
def test_b():
    assert False

@pytest.mark.dependency(depends=["test_a"])
def test_c():
    pass


class TestClass(object):

    @pytest.mark.dependency()
    def test_b(self):
        pass



import pytest

@pytest.mark.dependency()
@pytest.mark.xfail(reason="deliberate fail")
def test_a():
    assert False

@pytest.mark.dependency()
def test_b():
    pass

@pytest.mark.dependency(depends=["test_a"])
def test_c():
    pass

@pytest.mark.dependency(depends=["test_b"])
def test_d():
    pass

@pytest.mark.dependency(depends=["test_b", "test_c"])
def test_e():
    pass



"""Simple dependencies between tests.
"""

import pytest


def test_no_skip(ctestdir):
    """One test is skipped, but no other test depends on it,
    so all other tests pass.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pytest.skip("explicit skip")

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency(depends=["test_b"])
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=3, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a SKIPPED
        *::test_b PASSED
        *::test_c PASSED
        *::test_d PASSED
    """)


def test_skip_depend(ctestdir):
    """One test is skipped, other dependent tests are skipped as well.
    This also includes indirect dependencies.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pytest.skip("explicit skip")

        @pytest.mark.dependency(depends=["test_b"])
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1, skipped=3, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b SKIPPED
        *::test_c SKIPPED
        *::test_d SKIPPED
    """)


def test_fail_depend(ctestdir):
    """One test fails, other dependent tests are skipped.
    This also includes indirect dependencies.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            assert False

        @pytest.mark.dependency(depends=["test_b"])
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b FAILED
        *::test_c SKIPPED
        *::test_d SKIPPED
    """)


def test_named_fail_depend(ctestdir):
    """Same as test_fail_depend, but using custom test names.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency(name="a")
        def test_a():
            pass

        @pytest.mark.dependency(name="b")
        def test_b():
            assert False

        @pytest.mark.dependency(name="c", depends=["b"])
        def test_c():
            pass

        @pytest.mark.dependency(name="d", depends=["c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=1, skipped=2, failed=1)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b FAILED
        *::test_c SKIPPED
        *::test_d SKIPPED
    """)


def test_explicit_select(ctestdir):
    """Explicitly select only a single test that depends on another one.

    Since the other test has not been run at all, the selected test
    will be skipped.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_c"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose", "test_explicit_select.py::test_d")
    result.assert_outcomes(passed=0, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_d SKIPPED
    """)


def test_depend_unknown(ctestdir):
    """Depend on an unknown test that is not even defined in the test set.

    Note that is not an error to depend on an undefined test, but the
    dependent test will be skipped since the non-existent dependency
    has not been run successfully.
    """
    ctestdir.makepyfile("""
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_c():
            pass

        @pytest.mark.dependency(depends=["test_x"])
        def test_d():
            pass
    """)
    result = ctestdir.runpytest("--verbose")
    result.assert_outcomes(passed=3, skipped=1, failed=0)
    result.stdout.fnmatch_lines("""
        *::test_a PASSED
        *::test_b PASSED
        *::test_c PASSED
        *::test_d SKIPPED
    """)



#! /usr/bin/python
"""pytest-dependency - Manage dependencies of tests

This pytest plugin manages dependencies of tests.  It allows to mark
some tests as dependent from other tests.  These tests will then be
skipped if any of the dependencies did fail or has been skipped.
"""

import distutils.log
import os
import os.path
import re
import string
from setuptools import setup
import setuptools.command.sdist as st_sdist
try:
    import setuptools_scm
    version = setuptools_scm.get_version()
    with open(".version", "wt") as f:
        f.write(version)
except (ImportError, LookupError):
    try:
        with open(".version", "rt") as f:
            version = f.read()
    except (OSError, IOError):
        distutils.log.warn("warning: cannot determine version number")
        version = "UNKNOWN"


class sdist(st_sdist.sdist):
    def make_release_tree(self, base_dir, files):
        st_sdist.sdist.make_release_tree(self, base_dir, files)
        if not self.dry_run:
            src = "pytest_dependency.py"
            dest = os.path.join(base_dir, src)
            if hasattr(os, 'link') and os.path.exists(dest):
                os.unlink(dest)
            subst = {'DOC': __doc__, 'VERSION': version}
            with open(src, "rt") as sf, open(dest, "wt") as df:
                df.write(string.Template(sf.read()).substitute(subst))


setup(
    name='pytest-dependency',
    version=version,
    description='Manage dependencies of tests',
    author='Rolf Krahl',
    author_email='rolf@rotkraut.de',
    maintainer='Rolf Krahl',
    maintainer_email='rolf@rotkraut.de',
    url='https://github.com/RKrahl/pytest-dependency',
    license='Apache Software License 2.0',
    long_description=__doc__,
    project_urls={
        'Documentation': 'https://pytest-dependency.readthedocs.io/',
        'Source Code': 'https://github.com/RKrahl/pytest-dependency',
    },
    py_modules=['pytest_dependency'],
    install_requires=['pytest >= 3.6.0'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
    ],
    entry_points={
        'pytest11': [
            'dependency = pytest_dependency',
        ],
    },
    cmdclass = {'sdist': sdist},
)



import pytest


class TestClass(object):

    @pytest.mark.dependency()
    @pytest.mark.xfail(reason="deliberate fail")
    def test_a(self):
        assert False

    @pytest.mark.dependency()
    def test_b(self):
        pass

    @pytest.mark.dependency(depends=["TestClass::test_a"])
    def test_c(self):
        pass

    @pytest.mark.dependency(depends=["TestClass::test_b"])
    def test_d(self):
        pass

    @pytest.mark.dependency(depends=["TestClass::test_b", "TestClass::test_c"])
    def test_e(self):
        pass


class TestClassNamed(object):

    @pytest.mark.dependency(name="a")
    @pytest.mark.xfail(reason="deliberate fail")
    def test_a(self):
        assert False

    @pytest.mark.dependency(name="b")
    def test_b(self):
        pass

    @pytest.mark.dependency(name="c", depends=["a"])
    def test_c(self):
        pass

    @pytest.mark.dependency(name="d", depends=["b"])
    def test_d(self):
        pass

    @pytest.mark.dependency(name="e", depends=["b", "c"])
    def test_e(self):
        pass



import pytest

@pytest.mark.dependency()
@pytest.mark.xfail(reason="deliberate fail")
def test_a():
    assert False


class TestClass1(object):

    @pytest.mark.dependency()
    def test_b(self):
        pass


class TestClass2(object):

    @pytest.mark.dependency()
    def test_a(self):
        pass

    @pytest.mark.dependency(depends=["test_a"])
    def test_c(self):
        pass

    @pytest.mark.dependency(depends=["test_a"], scope='class')
    def test_d(self):
        pass

    @pytest.mark.dependency(depends=["test_b"], scope='class')
    def test_e(self):
        pass
